require("common.baseclass")
require("object.server.ServerObject")


ServerMgr = ServerMgr or BaseClass()

local defIp = "10.21.210.163"
local defPort = "5555"
local MaxRecentServerCount = 4  -- 最多保存4个最近服务器

function ServerMgr:__init()
	self.serverList = {}	
	self.recommendServerList = {}  --推荐的服务器列表
	
	self.resourceCheckUrl = ""		-- 资源更新url
	self.rechargeChannelUrl = ""	-- 充值通道url
	self.gameNoticeUrl = ""			-- 公告url
	self.gameNoticeListUrl = ""		-- 公告列表url
	self.servicesUrl = ""
	self.testVersion = ""			-- 启动测试分区的版本号条件
	self.testStaticUrl = ""			-- 测试分区的URL
	self.sdkVersion = ""			-- androidSDK版本号
	self.iosSdkVersion = ""			-- iosSDK版本号
end

-- 校验数据的合法性
function ServerMgr:checkData(jsonData)
	return jsonData and type(jsonData) == "table" and jsonData.serversList and type(jsonData.serversList) == "table"
end

function ServerMgr:parseData(jsonData)
	if not self:checkData(jsonData) then
		return false
	end
	
	self:setServerList(jsonData.serversList)
	self:setServerUrl(jsonData)	
	self:setGroupId(jsonData.groupId)	
	
	--print_r(jsonData,"jsonData : ")
	return true
end

-- 分区Id
function ServerMgr:setGroupId(groupId)
	self.groupId = groupId
end

function ServerMgr:getGroupId()
	return self.groupId
end

-- 是否获取到了服务器列表
function ServerMgr:isEmpty()
	return _G.next(self.serverList) == nil
end

function ServerMgr:getServerList()
	return self.serverList
end

function ServerMgr:clear()
	if self.serverList then
		for k,v in pairs(self.serverList) do
			v:DeleteMe()
		end
	end
	
	if self.recommendServerList then
		for k,v in pairs(self.recommendServerList) do
			v:DeleteMe()
		end
	end
	
	self.serverList = {}	
	self.recommendServerList = {}
end

function ServerMgr:getResourceCheckUrl()
	return self.resourceCheckUrl
end

function ServerMgr:getRechargeChannelUrl()
	return self.rechargeChannelUrl
end

function ServerMgr:getNoticeUrl()
	return self.gameNoticeUrl
end

function ServerMgr:getNoticeListUrl()
	return self.gameNoticeListUrl
end

function ServerMgr:getServicesUrl()
	return self.servicesUrl
end

function ServerMgr:getTestVersion()
	return self.testVersion
end

function ServerMgr:getTestStaticUrl()
	return self.testStaticUrl
end

function ServerMgr:getSDKVersion()
	if self.sdkVersion then
		return tonumber(self.sdkVersion)
	else
		return 0
	end		
end

function ServerMgr:getIosSDKVersion()
	if self.sdkVersion then
		return tonumber(self.iosSdkVersion)
	else
		return 0
	end	
end

-- 解析服务器的信息
function ServerMgr:setServerList(serverList)
	if not serverList then
		return
	end
	
	-- 先排序
	serverList = self:sortServerList(serverList)
	for k, v in pairs(serverList) do 
		local server = ServerObject.New()
		server:setPT(v)	
		if server:getServerRecommend()==1 then 		
			table.insert(self.recommendServerList, server)			
		end
		table.insert(self.serverList, server)
	end
end

-- 解析分区对应的各种url
function ServerMgr:setServerUrl(urlList)
	if urlList then
		self.resourceCheckUrl = urlList["resourceCheckUrl"]
		self.rechargeChannelUrl = urlList["rechargeChannelUrl"]
		self.gameNoticeUrl = urlList["gameNoticeUrl"]
		self.gameNoticeListUrl = urlList["gameNoticeListUrl"]
		self.servicesUrl = urlList["servicesUrl"]		
		self.testVersion = urlList["testVersion"]
		self.testStaticUrl = urlList["backupStaticServerAddr"]
		self.sdkVersion = urlList["sdkVersion"]
		self.iosSdkVersion = urlList["iosSdkVersion"]
	end
end

--[[function ServerMgr:getLastTimeLgoinServer()
	--首先读取上次登录的server
	local reader = CCUserDefault:sharedUserDefault()
	local ip = reader:getStringForKey(Config.UserDefaultKey.Server_LastTimeServerIp)
	local port = reader:getStringForKey(Config.UserDefaultKey.Server_LastTimeServerPort)	
	if ip ~= "" and port ~= "" then
		return self:getServer(ip, port)
	end		
	--没有最近登录从推荐服随机返回
	local server = self:getRecommendLoginServer()
	if server then
		return server
	end
	--最后返回一个可用的服务器
	local server = self:getCanUseServer()	

	return server
end--]]

--返回一个可用的服务器
function ServerMgr:getCanUseServer()
	for k, v in pairs(self.serverList) do 
		local state = v:getServerState()
		if state >= 2 then 
			return v
		end
	end
end

function ServerMgr:getRecommendServerList()
	return self.recommendServerList
end

function ServerMgr:getRecommendLoginServer()
	for k, v in pairs(self.recommendServerList) do 
		return v   --从推荐列表  随便返回一个
	end
end

function ServerMgr:getServer(ip, port)
	for k, v in pairs(self.serverList) do 
		local tmpIp = v:getServerIp()		
		local tmpPort = tostring(v:getServerPort())
		if ip==tmpIp and port==tmpPort then 	
			return v
		end
	end		
	--return self.serverList[1]   --找不到就返回最新的
end

--通过服务器ID返回serverobj（liu新增） 
function ServerMgr:getServerById(serverid)
	for k, v in pairs(self.serverList) do 
		local tmpserverid = tostring(v:getServerId())		
		if serverid==tmpserverid  then 	
			return v
		end
	end		
	--return self.serverList[1]   --找不到就返回最新的
end

function ServerMgr:getRecentServers()
	local reader = CCUserDefault:sharedUserDefault()
	local recentServers = reader:getStringForKey(Config.UserDefaultKey.Recent_Server)
	return recentServers
end


function ServerMgr:getRecentId()
	local reader = CCUserDefault:sharedUserDefault()
	local recentServers = reader:getStringForKey(Config.UserDefaultKey.Recent_Id)
	return recentServers
end

--保存上次登录的服务器
--保存的形式,以&隔开:  ip=10.11.11.11,port=8888&ip=10.12.111.111,port=3333&....
function ServerMgr:saveLastTimeServer(ip, port, serverId)
    --iprint(self.serverList)
    --local serverObj = self:getServer(ip, tostring(port))   

    --[[local serverId
    if serverObj then 
        serverId = serverObj:getServerId() or ""
        --CCLuaLog(serverId)
    end ]]   

	local writer = CCUserDefault:sharedUserDefault()
	local recentServers = writer:getStringForKey(Config.UserDefaultKey.Recent_Server)
	if recentServers and string.len(recentServers)>0 then 	
		recentServers = string.gsub(recentServers, "ip="..ip..",port="..port.."&", "")		
		
	end
	recentServers = recentServers or ""
	local latestServer = "ip="..ip..",port="..port
	recentServers = latestServer .. "&" .. recentServers
	
	local servers = string.split(recentServers, "&")
	if servers and table.size(servers) > MaxRecentServerCount then 
		recentServers = ""
		for i=1, MaxRecentServerCount do
			if servers[i] then 
				recentServers = recentServers .. servers[i].. "&"				
			end
		end
	end
	

	if recentServers ~= writer:getStringForKey(Config.UserDefaultKey.Recent_Server) then 
		writer:setStringForKey(Config.UserDefaultKey.Recent_Server, recentServers)
		writer:flush()
	end	

    local recentId = writer:getStringForKey(Config.UserDefaultKey.Recent_Id)
	if recentId and string.len(recentId)>0 then 	
		recentId = string.gsub(recentId, "serverid="..serverId.."&", "")
	end
	

	recentId = recentId or ""
	local latestServer = "serverid="..serverId
	recentId = latestServer .. "&" .. recentId
	
	local servers = string.split(recentId, "&")
	if servers and table.size(servers) > MaxRecentServerCount then 
		recentId = ""
		for i=1, MaxRecentServerCount do
			if servers[i] then 
				recentId = recentId .. servers[i].. "&"				
			end
		end
	end

	--CCLuaLog(recentId)
	--CCLuaLog("--------------------")
	--CCLuaLog(writer:getStringForKey(Config.UserDefaultKey.Recent_Name))
     
    if recentId ~= writer:getStringForKey(Config.UserDefaultKey.Recent_Id) then 
		writer:setStringForKey(Config.UserDefaultKey.Recent_Id, recentId)
		writer:flush()
	end	
end


--IP等于ID，后来改了模式
function ServerMgr:removeRecentServer(ip, port)
	local writer = CCUserDefault:sharedUserDefault()	
	iprint("ip:",ip,"port:",port)
	if ip and port then 	
		local recentServers = writer:getStringForKey(Config.UserDefaultKey.Recent_Server)
		if recentServers and string.len(recentServers)>0 then 	
			recentServers = string.gsub(recentServers, "ip="..ip..",port="..port.."&", "")		
		end		
		
		if recentServers ~= writer:getStringForKey(Config.UserDefaultKey.Recent_Server) then 
			writer:setStringForKey(Config.UserDefaultKey.Recent_Server, recentServers)
			writer:flush()
		end
	elseif ip then
		local recentServers = writer:getStringForKey(Config.UserDefaultKey.Recent_Id)
		if recentServers and string.len(recentServers)>0 then 	
			iprint("recentServers",recentServers)
			recentServers = string.gsub(recentServers, "serverid="..ip.."&", "")	
			iprint("recentServers",recentServers)	
		end		
		
		if recentServers ~= writer:getStringForKey(Config.UserDefaultKey.Recent_Id) then 
			writer:setStringForKey(Config.UserDefaultKey.Recent_Id, recentServers)
			writer:flush()
		end
	else  
		writer:setStringForKey(Config.UserDefaultKey.Recent_Server, "")
		writer:flush()
	end
end

function ServerMgr:getServerByShowOrder(order)
	for k, v in pairs(self.serverList) do 
		if v:getShowOrders() == order then 
			return v
		end
	end
end	

function ServerMgr:getSelectServer()
	return self.selectServer
end

function ServerMgr:setSelectServer(ip, port, serverId)
	for k, v in pairs(self.serverList) do 
		if v:getServerIp() == ip and v:getServerPort() == port and v:getServerId() == serverId then 
			self.selectServer = v    
			break  
		end
	end
end

function ServerMgr:sortServerList(servers)
	local order = {}
	local serverCnt = table.size(servers)
	
	--根据排序规则， 对服务器列表排序
	local sortByStartTime = function (a, b)
		if a.openTime and b.openTime then
			if a.openTime > b.openTime then
				return true
			elseif a.openTime == b.openTime then
				return a.orders > b.orders
			end
		else
			return a.orders > b.orders
		end
	end
	table.sort(servers, sortByStartTime)
	
	--添加显示的顺序
	for index, serverTable in pairs(servers) do
		serverTable.showOrders = serverCnt
		serverCnt = serverCnt - 1
	end
				
	return servers	
end

function ServerMgr:login(server)
	if server then 
		local serverIp = server:getServerIp()
		local serverPort = server:getServerPort()
		local serverId = server:getServerId()			
		LoginWorld.Instance:getLoginManager():connectServer(serverIp, serverPort, serverId)						
	end		
end

function ServerMgr:handleServerLogin(server)
	if server then		
		LoginWorld.Instance:getStatisticsMgr():requestStepStatistics(GameStep.StartEnterGame)
		local serverId = server:getServerId()	
		local notifyMgr = LoginWorld.Instance:getNotifyManager()
		notifyMgr:requireServerNotifyUrlList(serverId)		
		local state = server:getServerState()	
		
		-- 白名单机制, 方便服务器更新提前进入
		local keyFilePath = SFGameHelper:getExtStoragePath() .. "/newBeeTes.key"
		local ignoreState = CCFileUtils:sharedFileUtils():isFileExist(keyFilePath)
		
		if not ignoreState and ( state==-1 or state==0 or state==1) then	--服务器处于维护状态										
			--UIManager.Instance:showSystemTips(Config.LoginWords[8505]..server:getStartTime())
			notifyMgr:setNeedShowMaintainView(true)
		else
			local serverMgr = LoginWorld.Instance:getServerMgr()
			local loginMgr = LoginWorld.Instance:getLoginManager()			
			self:login(server)
			notifyMgr:setNeedShowMaintainView(false)
		end
	end		
end